
% Copyright 2013, Yi Yu <yuy@comp.nus.edu.sg>
% This program is for Personal Music Video Generation based on Modeling Scene Moods and is free: 
% You can redistribute it and/or modify
% it under the terms of the GNU General Public License.



function annotationInfo = funcGetAnAnnotation(annoFileName, geoTagSet, moodTagSet, usePrimaryMoodOnly) %#codegen

    thisAnnotationInfo = textread(annoFileName, '%s', 'delimiter', '\n');
    geoTagFeat = zeros(numel(thisAnnotationInfo), numel(geoTagSet));
    moodIndex  = zeros(numel(thisAnnotationInfo), 1);
    startTime  = zeros(numel(thisAnnotationInfo), 1);
    endTime    = zeros(numel(thisAnnotationInfo), 1);
    for aIdx=1:numel(thisAnnotationInfo)
        segDetail = regexp(thisAnnotationInfo{aIdx}, ';', 'split');

        % get geo-tags per segment
        geoTagStr = segDetail{1};
        geoTagInfo = regexp(geoTagStr, ',', 'split');
        geoScore = zeros(numel(geoTagInfo)/2,2);
        geoTagSav = '';
        for k=1:2:numel(geoTagInfo)
            % index = strfind(geoTagSet, lower(strtrim(geoTagInfo{k})));
            % geoScore((k+1)/2,1) = find(not(cellfun('isempty', index)));
            index = strcmp(geoTagSet, lower(strtrim(geoTagInfo{k})));
            geoScore((k+1)/2,1) = find(index);
            geoScore((k+1)/2,2) = str2num(geoTagInfo{k+1});
            geoTagSav = [geoTagSav ';' lower(strtrim(geoTagInfo{k}))];
        end;
        % fprintf('geoTag %3d: %s\n', fIdx, geoTagSav);
        % store geo-tag and score
        geoScore = sortrows(geoScore, 2);
        geoScore(:,2) = 1./(1:size(geoScore,1)); % geoScore(1,2)./geoScore(:,2);
        index = geoScore(:,1);
        score = zeros(1,numel(geoTagSet));
        score(index) = geoScore(:,2);
        geoTagFeat(aIdx,:) = score;

        % get mood-tags per segment
        moodTagStr = segDetail{2};
        moodTagInfo = regexp(moodTagStr, ',', 'split');
        moodScore = zeros(numel(moodTagInfo),2);
        moodTagSav = '';
        for k=1:numel(moodTagInfo)
            % index = strfind(moodTagSet, lower(strtrim(moodTagInfo{k})));
            % moodScore(k,1) = find(not(cellfun('isempty', index)));
            index = strcmp(moodTagSet, lower(strtrim(moodTagInfo{k})));
            moodScore(k,1) = find(index);
            moodScore(k,2) = 1/k;
            moodTagSav = [moodTagSav ';' lower(strtrim(moodTagInfo{k}))];
            if usePrimaryMoodOnly
                % currently only consider the first mood tag
                break;
            end;
        end;
        % fprintf('moodTag%3d: %s\n', fIdx, moodTagSav); 

        if usePrimaryMoodOnly
            % currently only consider the first mood tag
            moodIndex(aIdx) = moodScore(1,1);
        else
            fprintf(2, 'Undefined operation\n');
            return;
        end;

        % get start/end time
        startTime(aIdx) = str2num(segDetail{3});
        endTime(aIdx)   = str2num(segDetail{4});
    end;

    annotationInfo.geoTagFeat = geoTagFeat;
    annotationInfo.moodIndex  = moodIndex;
    annotationInfo.startTime  = startTime;
    annotationInfo.endTime    = endTime;
