
% Copyright 2013, Yi Yu <yuy@comp.nus.edu.sg>
% This program is for Personal Music Video Generation based on Modeling Scene Moods and is free: 
% You can redistribute it and/or modify
% it under the terms of the GNU General Public License.



function recommend = funcMusicRecommend(videoAnnoFile,videoMpgFile, config, geoTagSet, moodTagSet, musicDb, musicBucket) %#codegen

% ----------- 5. music recommendation -------------------------------------
fprintf('Recommend music for (%s,%s)\n', videoAnnoFile,videoMpgFile);


thisVideoAnnot = funcGetAnAnnotation(videoAnnoFile, geoTagSet, moodTagSet, config.usePrimaryMoodOnly);
thisVideoFeat  = funcGetAVisualFeat(videoMpgFile);

% synchronization between geo-feature and visual feature
numKeyFrm = find(thisVideoFeat.frameTime <= thisVideoAnnot.endTime(end), 1, 'last');
geoTagFeat = zeros(numKeyFrm, size(thisVideoAnnot.geoTagFeat,2));
moodIndex  = zeros(numKeyFrm, 1);
numItem = 0;
for sIdx=1:numel(thisVideoAnnot.startTime)
    index = thisVideoFeat.frameTime >= thisVideoAnnot.startTime(sIdx) ...
          & thisVideoFeat.frameTime <  thisVideoAnnot.endTime(sIdx);
    numThisSeg = sum(index);
    geoTagFeat(numItem+1:numItem+numThisSeg,:) = repmat(thisVideoAnnot.geoTagFeat(sIdx,:), numThisSeg, 1);
    moodIndex( numItem+1:numItem+numThisSeg,:) = repmat(thisVideoAnnot.moodIndex( sIdx,:), numThisSeg, 1);
    numItem = numItem + numThisSeg;
end;
thisVisualFeat = thisVideoFeat.visualFeat(1:numKeyFrm,:);


% 5.2 get the trained model
[emitWeight, transWeight] = getSvmModelInfo(config.trainModelFile);
svmhmm.emitW = emitWeight;
svmhmm.transW= transWeight;


% 5.3 find 'uniqMood' (top-3?) with 'geoTagFeat', 'thisVisualFeat';
extFeat = [];
if config.useGeoFeat
    extFeat = [extFeat geoTagFeat];
end;
if config.useVisualFeat
    extFeat = [extFeat thisVisualFeat];
end;
recogMood = viterbi_svm_Nbest(svmhmm, extFeat);
histInfo = getHistInfo(recogMood);
numFoundMood = min(size(histInfo,1), 3);
uniqMood = histInfo(1:numFoundMood,1);
fprintf('Recognized moods for the video: ');
for k=1:numFoundMood
    fprintf('%s', moodTagSet{uniqMood(k)});
end;
fprintf('\n');

% 5.4 get 'songInfo' with 'uniqMood'
candiSongs = [];
for mIdx=1:numel(uniqMood)
    moodIndex = uniqMood(mIdx);
    candiSongs = [candiSongs; musicBucket(moodIndex).bucket];
end;
uniqSong = unique(candiSongs(:,1));
songInfo = zeros(numel(uniqSong),2);
songInfo(:,1) = uniqSong;
for k=1:numel(uniqSong)
    index = candiSongs(:,1)==uniqSong(k);
    songInfo(k,2) = sum(candiSongs(index,2));
end;

finalScore = sortrows(songInfo,2);
finalScore = finalScore(end:-1:1,:);

% final result: songIdxSet
numRecommendSong = min(size(finalScore,1),3);
recommend.songIdxSet = finalScore(1:numRecommendSong,1);

% fprintf('Recommended songs:\n');
% for k=1:numRecommendSong
%     audioIdx = recommend.songIdxSet(k);
%     annoMoodTagStr = '';
%     for n=1:size(musicDb(audioIdx).moodScore,1)
%         if musicDb(audioIdx).moodScore(n,1)~=0
%             annoMoodTagStr = [annoMoodTagStr  moodTagSet{musicDb(audioIdx).moodScore(n,1)} ','];
%         end;
%     end;
%     fprintf('    SongIdx=%3d, annotatedMood=%s\n        SongName=%s\n', ...
%         audioIdx, annoMoodTagStr, musicDb(audioIdx).trackName);
% end;
