% Copyright 2013, Yi Yu <yuy@comp.nus.edu.sg>
% This program is for Personal Music Video Generation based on Modeling Scene Moods and is free: 
% You can redistribute it and/or modify
% it under the terms of the GNU General Public License.


function similarity = getAudioSimilarity(feat1, feat2)
    newFeat1 = getNewFeat(feat1);
    newFeat1 = newFeat1 - repmat(mean(newFeat1,1),size(newFeat1,1),1);

    newFeat2 = getNewFeat(feat2);
    newFeat2 = newFeat2 - repmat(mean(newFeat2,1),size(newFeat2,1),1);

    similarity = sum(newFeat1.*newFeat2,1) ./ sqrt(sum(newFeat1.^2,1) .* sum(newFeat2.^2,1) + 1e-8);


function newFeat = getNewFeat(feat)
    mfccMean = mean(feat.mfcc,2);
    covar    = cov(feat.mfcc');
	mfccStd  = sqrt(diag(covar));

    minNoteIndex =  21;
    maxNoteIndex = 108;
    pitchHist = zeros(88,1);
    for k=minNoteIndex:maxNoteIndex
        pitchHist(k-minNoteIndex+1) = sum(feat.pitch==k);
    end;

    newFeat = [mfccMean/sum(mfccMean); mfccStd/sum(mfccStd); pitchHist/sum(pitchHist)];
